// PV227, Lesson 10: Microfacets
#pragma once

#include "../../Framework/PV227.h"

using namespace std;
using namespace PV227;

//---------------------
//----    SCENE    ----
//---------------------

// Shader programs
ShaderProgram nolit_program;
ShaderProgram BlinnPhongOriginal_program;
ShaderProgram BlinnPhongNormalized_program;
ShaderProgram CookTorrance_program;
ShaderProgram OrenNayar_program;
ShaderProgram AshikhminShirley_program;

// Geometries we use in this lecture
Geometry geom_cube;
Geometry geom_sphere;
Geometry geom_torus;
Geometry geom_cylinder;
Geometry geom_capsule;
Geometry geom_teapot;

// A camera that allows us to look at the object from different views
SimpleCamera the_camera;

// Data of our materials
MaterialData_UBO WhiteMaterial_ubo;

// Position of the central object and the light
ModelData_UBO CentralObjectModel_ubo;
ModelData_UBO LightModel_ubo;

// Data of all our lights - positions, colors, etc.
PhongLightsData_UBO PhongLights_ubo;

// Data of our camera - view matrix, projection matrix, etc.
CameraData_UBO CameraData_ubo;

// Functions that works with scene objects
void reload_shaders();
void init_scene();
void update_scene(int app_time_diff_ms);
void render_scene();

//-------------------
//----    GUI    ----
//-------------------

// Main AntTweakBar object
TwBar *the_gui;

// Variables that are changed with GUI
float light_pos;
float light_intensity;
float ambient_light_intensity;

glm::vec3 material_diffuse;
glm::vec3 material_fresnel;
glm::vec3 material_specular;
float material_roughness;
float material_shininess;
float material_shininess_tangent;
float material_shininess_bitangent;

int material_preset;
const int MATERIAL_PRESET_USER = 0;				// User values
const int MATERIAL_PRESET_PLASTIC_USER = 1;		// Fresnel = 0.04, user diffuse
const int MATERIAL_PRESET_PLASTIC_RED = 2;		// Fresnel = 0.04, red diffuse
const int MATERIAL_PRESET_PLASTIC_GREEN = 3;	// Fresnel = 0.04, green diffuse
const int MATERIAL_PRESET_PLASTIC_BLUE = 4;		// Fresnel = 0.04, blue diffuse
const int MATERIAL_PRESET_IRON = 5;				// Fresnel = (0.56, 0.57, 0.58), diffuse = 0.0
const int MATERIAL_PRESET_COPPER = 6;			// Fresnel = (0.95, 0.64, 0.54), diffuse = 0.0
const int MATERIAL_PRESET_GOLD = 7;				// Fresnel = (1.00, 0.71, 0.29), diffuse = 0.0
const int MATERIAL_PRESET_ALUMINIUM = 8;		// Fresnel = (0.91, 0.92, 0.92), diffuse = 0.0
const int MATERIAL_PRESET_SILVER = 9;			// Fresnel = (0.95, 0.93, 0.88), diffuse = 0.0

int central_object;
const int CENTRAL_OBJECT_CUBE = 0;
const int CENTRAL_OBJECT_SPHERE = 1;
const int CENTRAL_OBJECT_TORUS = 2;
const int CENTRAL_OBJECT_CYLINDER = 3;
const int CENTRAL_OBJECT_CAPSULE = 4;
const int CENTRAL_OBJECT_TEAPOT = 5;

int material_type;
const int MATERIAL_TYPE_BLINN_PHONG_ORIGINAL = 0;
const int MATERIAL_TYPE_BLINN_PHONG_NORMALIZED = 1;
const int MATERIAL_TYPE_COOK_TORRANCE = 2;
const int MATERIAL_TYPE_OREN_NAYAR = 3;
const int MATERIAL_TYPE_ASHIKHMIN_SHIRLEY = 4;

// Callbacks from the GUI
void TW_CALL reload(void *);

// Functions that works with GUI
void init_gui();

//---------------------------
//----    APPLICATION    ----

// Current window size
int win_width = 640;
int win_height = 480;

// Current time of the application in milliseconds, for animations
int app_time_ms = 0;
int last_glut_time = 0;
